/*****************************************************************************

	GTK Development Environment
	
	Copyright (c) 2000 End of the World Software

	date:	December 05 2000
	author: Gertjan Zwartjes
	email:	g.zwartjes@student.tue.nl

  **************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "gde.h"


/*---------------------------------------------------------

	file_list_new()

	Creates a new file list.

---------------------------------------------------------*/
GdeFileList *file_list_new(void)
{
	GdeFileList* file_list;
	
	file_list = g_malloc(sizeof(GdeFileList));
	file_list->filename = NULL;
	file_list->tree_item = NULL;
	file_list->modified = (time_t)(-1);
	file_list->prev_file = NULL;
	file_list->next_file = NULL;

	return file_list;
}


/*---------------------------------------------------------

	file_list_new_item()

	Adds a new item to the file list and returns a
	pointer to the new element.

---------------------------------------------------------*/
GdeFileList *file_list_new_item(GdeFileList *file_list)
{
	while (file_list->next_file != NULL)
		file_list = (GdeFileList*) file_list->next_file;
		
	(GdeFileList*) file_list->next_file = file_list_new();
	(GdeFileList*) ((GdeFileList*) file_list->next_file)->prev_file = file_list;
	
	return (GdeFileList*) file_list->next_file;
}


/*---------------------------------------------------------

	file_list_find_node()

	Find the file entry in a specific file list, which
	belongs to a specific node. Returns NULL if it is
	not in the list.

---------------------------------------------------------*/
GdeFileList *file_list_find_node(GdeFileList *file_list, GtkCTreeNode *node)
{
	while ((file_list->tree_item != node) && (file_list->next_file != NULL))
		file_list = (GdeFileList*) file_list->next_file;

	if (file_list->tree_item == node)
		return file_list;
	else
		return NULL;
}


/*---------------------------------------------------------

	file_list_save()

	Writes lines with the string opt and each of the files
	in the list behind them to the specified file.

---------------------------------------------------------*/
void file_list_save(FILE *file, GdeFileList *file_list, const gchar *opt)
{
	while (file_list->next_file) {
		file_list = (GdeFileList *) file_list->next_file;
		fprintf(file, "%s%s\n", opt, file_list->filename);
	}
}


/*---------------------------------------------------------

	file_list_clear()

	Clear the given file list.
	
---------------------------------------------------------*/
void file_list_clear(GdeFileList *file_list)
{
	GdeFileList *next_file;

	next_file = (GdeFileList *) file_list->next_file;
	file_list->next_file = NULL;
	
	while (next_file) {
		file_list = next_file;
		proj_win_remove_file(file_list->tree_item);
		g_free(file_list->filename);
		next_file = (GdeFileList *) file_list->next_file;
		g_free(file_list);
	}
}


/*----------------------------------------------------------

	file_list_modified()

	Checks wether one of the files in file_list have a
	date newer than the date we stored. If so, return
	true.

----------------------------------------------------------*/
gboolean file_list_modified(GdeFileList *file_list)
{
	struct stat file_stat;	
	
	while (file_list) {
		if (file_list->filename) {
			if (stat(file_list->filename, &file_stat) == -1)
				g_error("Could not get file stats: %s", g_strerror(errno));

			if (file_stat.st_mtime > file_list->modified)
				return TRUE;
		}

		file_list = (GdeFileList *) file_list->next_file;
	}

	return FALSE;
}


/*----------------------------------------------------------

	file_list_modified_reset()

	Reset modified to all `file' values.

----------------------------------------------------------*/
void file_list_modified_reset(GdeFileList *file_list)
{
	struct stat file_stat;

	while (file_list) {
		if (file_list->filename) {
			if (stat(file_list->filename, &file_stat) == -1)
				g_error("Could not get file stats: %s", g_strerror(errno));

			file_list->modified = file_stat.st_mtime;
		}
		
		file_list = (GdeFileList *) file_list->next_file;
	}
}
