/*****************************************************************************

	GTK Development Environment
	
	Copyright (c) 2000 End of the World Software

	date:	November 30 2000
	author: Gertjan Zwartjes
	email:	g.zwartjes@student.tue.nl

  **************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "gde.h"


/*=========================================================

	Variable Declarations

=========================================================*/
static GtkWidget *file_dialog;
static gboolean file_sel_ok = FALSE;
static gchar old_dirname[512];


/*---------------------------------------------------------

	file_win_ok()

	Callback for the file selector ok button.

---------------------------------------------------------*/
static void file_win_ok(GtkWidget *widget, gpointer data)
{
	file_sel_ok = TRUE;
	gtk_widget_hide(GTK_WIDGET (file_dialog));
}


/*---------------------------------------------------------

	file_win_cancel()

	Callback fro the file dialog cancel button.

---------------------------------------------------------*/
static gint file_win_cancel(GtkWidget *widget, gpointer data)
{
	file_sel_ok = FALSE;
	gtk_widget_hide(GTK_WIDGET (file_dialog));
	return TRUE;
}


/*---------------------------------------------------------

	file_win()

	Returns a pointer to a GList containing the selected 
	filename(s), NULL otherwise. The GList is newly
	allocated, so it has to be freed, as the strings in
	it (which is the data pointer).

---------------------------------------------------------*/
GList *file_win(const gchar* title, FileWinSelectType file_win_select_type)
{
	GList *file_list;
	GList *file_sellist;
	gchar *filename;
	gchar *filepath;
	gchar *dirname;

	file_dialog = gtk_file_selection_new(title);

	if (*old_dirname) {
		dirname = g_strconcat(old_dirname, G_DIR_SEPARATOR_S, NULL);
		gtk_file_selection_set_filename(GTK_FILE_SELECTION (file_dialog), dirname);
		g_free(dirname);
	}
	
	gtk_window_set_transient_for(GTK_WINDOW (file_dialog), GTK_WINDOW (main_win));
	gtk_window_set_modal(GTK_WINDOW (file_dialog), TRUE);
	gtk_container_border_width(GTK_CONTAINER (file_dialog), 4);
	gtk_signal_connect(GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
					   "clicked", GTK_SIGNAL_FUNC (file_win_ok), NULL);
	gtk_signal_connect(GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->cancel_button),
					   "clicked", GTK_SIGNAL_FUNC (file_win_cancel), NULL);
	gtk_signal_connect(GTK_OBJECT (file_dialog), "delete_event",
					   GTK_SIGNAL_FUNC(file_win_cancel), NULL);
	gtk_window_position(GTK_WINDOW (file_dialog), GTK_WIN_POS_MOUSE);

	if (file_win_select_type == FILE_WIN_SELECT_MULTIPLE)
		gtk_clist_set_selection_mode(GTK_CLIST (GTK_FILE_SELECTION (file_dialog)->file_list), GTK_SELECTION_EXTENDED);

	gtk_widget_show(file_dialog);
	
	while (GTK_WIDGET_VISIBLE (file_dialog))
		gtk_main_iteration_do(TRUE);

	dirname = g_dirname(gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_dialog)));
	strcpy(old_dirname, dirname);
	g_free(dirname);
	
	file_list = NULL;
	if (file_sel_ok) {
		file_sellist = GTK_CLIST (GTK_FILE_SELECTION (file_dialog)->file_list)->selection;

		if ((file_win_select_type == FILE_WIN_SELECT_SINGLE) || ((file_win_select_type == FILE_WIN_SELECT_MULTIPLE) && (!file_sellist))) {
			filepath = gtk_file_selection_get_filename(GTK_FILE_SELECTION (file_dialog));
			if (*g_basename(filepath))
				file_list = g_list_append(file_list, g_strdup(filepath));	
		}
		
		if (file_win_select_type == FILE_WIN_SELECT_MULTIPLE) {
			dirname = g_dirname(gtk_file_selection_get_filename(GTK_FILE_SELECTION (file_dialog)));
			
			while (file_sellist) {
				gtk_clist_get_text(GTK_CLIST (GTK_FILE_SELECTION (file_dialog)->file_list), GPOINTER_TO_INT (file_sellist->data), 0, &filename);
				filepath = g_strconcat(dirname, G_DIR_SEPARATOR_S, filename, NULL);
				file_list = g_list_append(file_list, filepath);
				file_sellist = g_list_next(file_sellist);
			}

			g_free(dirname);
		}
	}
	
	gtk_widget_destroy(file_dialog);
	return file_list;
}
