/*****************************************************************************

	GTK Development Environment
	
	Copyright (c) 2000 End of the World Software

	date:	December 01 2000
	author: Gertjan Zwartjes
	email:	g.zwartjes@student.tue.nl

  **************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "gde.h"


/*==========================================================

	Variable Declaration

==========================================================*/
GdeRC *gde_rc;


/*---------------------------------------------------------

	gde_usage()

	Prints the usage message.

---------------------------------------------------------*/
static void gde_print_usage(void)
{
	g_print("Usage: gde [options] [file]\n\n");
	g_print("Options:\n");
	g_print("  -h, --help       Display command line options and exit\n");
	g_print("  -v, --version    Display version information and exit\n");
	g_print("\n");
	exit(0);
}


/*---------------------------------------------------------

	gde_print_version()

	Prints the current version number of GDE.

---------------------------------------------------------*/
static void gde_print_version(void)
{
	g_print("gde %s\n", GDE_VERSION);
	exit(0);
}


/*----------------------------------------------------------

	gde_write_rc()

	Writes settings to "~/.gderc".

----------------------------------------------------------*/
void gde_write_rc() 
{
	gchar *filename;
	FILE *file;
	gint i;
	GdeWindowInfo* win_info;

	filename = g_strconcat(g_get_home_dir(), G_DIR_SEPARATOR_S, GDE_RC, NULL);
	file = fopen(filename, "w");
	if (file) {
		fprintf(file, "%c Gtk Development Environment %s\n\n", COMMENT, GDE_VERSION);
		
		fprintf(file, "%c Recent file list\n", COMMENT);
		for (i = gde_rc->recent_size-1; i >= 0; i--) {
			if (*gde_rc->recent_list[i])
				fprintf(file, "recent_file=%s\n", gde_rc->recent_list[i]);
		}
	
		fprintf(file, "\n%c Messages Window fonts\n", COMMENT);
		fprintf(file, "info_font=%s\n", gde_rc->info_font);
		fprintf(file, "output_font=%s\n", gde_rc->output_font);
		fprintf(file, "error_font=%s\n", gde_rc->error_font);
	
		fprintf(file, "\n%c Save window sizes/positions on exit\n", COMMENT);
		fprintf(file, "save_window_info=%i\n", gde_rc->save_window_info);

		if (gde_rc->save_window_info) {
			win_info = g_malloc(sizeof(GdeWindowInfo));
			
			fprintf(file, "\n%c Main Window Info\n", COMMENT);
			main_win_get_window_info(win_info);
			fprintf(file, "main_win_info_x=%i\n", win_info->x);
			fprintf(file, "main_win_info_y=%i\n", win_info->y);
			fprintf(file, "main_win_info_width=%i\n", win_info->width);
			fprintf(file, "main_win_info_height=%i\n", win_info->height);
			fprintf(file, "main_win_info_visible=%i\n", win_info->visible);
			
			fprintf(file, "\n%c Project Explorer Window Info\n", COMMENT);
			proj_win_get_window_info(win_info);
			fprintf(file, "proj_win_info_x=%i\n", win_info->x);
			fprintf(file, "proj_win_info_y=%i\n", win_info->y);
			fprintf(file, "proj_win_info_width=%i\n", win_info->width);
			fprintf(file, "proj_win_info_height=%i\n", win_info->height);
			fprintf(file, "proj_win_info_visible=%i\n", win_info->visible);
			
			fprintf(file, "\n%c Messages Window Info\n", COMMENT);
			status_win_get_window_info(win_info);
			fprintf(file, "status_win_info_x=%i\n", win_info->x);
			fprintf(file, "status_win_info_y=%i\n", win_info->y);
			fprintf(file, "status_win_info_width=%i\n", win_info->width);
			fprintf(file, "status_win_info_height=%i\n", win_info->height);
			fprintf(file, "status_win_info_visible=%i\n", win_info->visible);

			g_free(win_info);
		}
		
		fprintf(file, "\n%c Recent file list size\n", COMMENT);
		fprintf(file, "recent_size=%i\n", gde_rc->recent_size);

		fprintf(file, "\n%c Open last opened file on startup\n", COMMENT);
		fprintf(file, "open_last_opened=%i\n", gde_rc->open_last_opened);
		fprintf(file, "last_opened=%s\n", gde_rc->last_opened);

		fclose(file);
	}
	else
		g_warning("Could not open '%s' for writing", filename);
		
	g_free(filename);
	g_free(gde_rc);
}


/*----------------------------------------------------------

	gde_read_rc()

	Read settings from "~/.gderc".

----------------------------------------------------------*/
static void gde_read_rc()
{
	gchar *filename;
	FILE *file;
	gchar buffer[512];
	gint i;
	GdeWindowInfo *main_win_info, *proj_win_info, *status_win_info;

	gde_rc = g_malloc(sizeof(GdeRC));
	
	/* Set everything to defaults */
	for (i = 0; i < GDE_RECENT_MAX; i++)
		gde_rc->recent_list[i][0] = 0;
	main_win_recent_clear();

	gde_rc->info_font[0] = 0;
	gde_rc->output_font[0] = 0;
	gde_rc->error_font[0] = 0;

	gde_rc->open_last_opened = FALSE;
	gde_rc->last_opened[0] = 0;
	
	gde_rc->save_window_info = TRUE;
	gde_rc->recent_size = 4;

	/* Read .gderc */
	filename = g_strconcat(g_get_home_dir(), G_DIR_SEPARATOR_S, GDE_RC, NULL);
	file = fopen(filename, "r");
	if (file) {
		main_win_info = g_malloc0(sizeof(GdeWindowInfo));
		proj_win_info = g_malloc0(sizeof(GdeWindowInfo));
		status_win_info = g_malloc0(sizeof(GdeWindowInfo));
		main_win_info->visible = TRUE;
		proj_win_info->visible = TRUE;
		status_win_info->visible = TRUE;
	
		while (!feof(file)) {
			memset(buffer, 0, sizeof(buffer));
			fgets(buffer, sizeof(buffer)-1, file);
			g_strchug(buffer);

			if ((*buffer) && (buffer[0] != COMMENT)) {
				if (str_get_value_of("recent_file=", buffer))
					main_win_recent_add(buffer);
		
				if (str_get_value_of("info_font=", buffer))
					strcpy(gde_rc->info_font, buffer);
				if (str_get_value_of("output_font=", buffer))
					strcpy(gde_rc->output_font, buffer);
				if (str_get_value_of("error_font=", buffer))
					strcpy(gde_rc->error_font, buffer);
				
				if (str_get_value_of("save_window_info=", buffer))
					gde_rc->save_window_info = strtol(buffer, NULL, 10);
				if (str_get_value_of("recent_size=", buffer))
					gde_rc->recent_size = strtol(buffer, NULL, 10);

				if (str_get_value_of("main_win_info_x=", buffer))
					main_win_info->x = strtol(buffer, NULL, 10);
				if (str_get_value_of("main_win_info_y=", buffer))
					main_win_info->y = strtol(buffer, NULL, 10);
				if (str_get_value_of("main_win_info_width=", buffer))
					main_win_info->width = strtol(buffer, NULL, 10);
				if (str_get_value_of("main_win_info_height=", buffer))
					main_win_info->height = strtol(buffer, NULL, 10);
				if (str_get_value_of("main_win_info_visible=", buffer))
					main_win_info->visible = strtol(buffer, NULL, 10);

				if (str_get_value_of("proj_win_info_x=", buffer))
					proj_win_info->x = strtol(buffer, NULL, 10);
				if (str_get_value_of("proj_win_info_y=", buffer))
					proj_win_info->y = strtol(buffer, NULL, 10);
				if (str_get_value_of("proj_win_info_width=", buffer))
					proj_win_info->width = strtol(buffer, NULL, 10);
				if (str_get_value_of("proj_win_info_height=", buffer))
					proj_win_info->height = strtol(buffer, NULL, 10);
				if (str_get_value_of("proj_win_info_visible=", buffer))
					proj_win_info->visible = strtol(buffer, NULL, 10);

				if (str_get_value_of("status_win_info_x=", buffer))
					status_win_info->x = strtol(buffer, NULL, 10);
				if (str_get_value_of("status_win_info_y=", buffer))
					status_win_info->y = strtol(buffer, NULL, 10);
				if (str_get_value_of("status_win_info_width=", buffer))
					status_win_info->width = strtol(buffer, NULL, 10);
				if (str_get_value_of("status_win_info_height=", buffer))
					status_win_info->height = strtol(buffer, NULL, 10);
				if (str_get_value_of("status_win_info_visible=", buffer))
					status_win_info->visible = strtol(buffer, NULL, 10);
					
				if (str_get_value_of("open_last_opened=", buffer))
					gde_rc->open_last_opened = strtol(buffer, NULL, 10);
				if (str_get_value_of("last_opened=", buffer))
					strcpy(gde_rc->last_opened, buffer);
			}
		}

		if (gde_rc->save_window_info) {
			main_win_set_window_info(main_win_info);
			proj_win_set_window_info(proj_win_info);
			status_win_set_window_info(status_win_info);
		}
		
		g_free(main_win_info);
		g_free(proj_win_info);
		g_free(status_win_info);
		
		fclose(file);
	}
	else 
		g_warning("Could not open '%s' for reading, using defaults", filename);

	g_free(filename);
}


/*----------------------------------------------------------

	gde_handle_sigchld()

	This is a handler for the SIGCHLD signal, which is
	signaled when a child process terminates or stops.
	Ask for its status information, to free a proc.

----------------------------------------------------------*/
void gde_handle_sigchld(int sig)
{
	int status;
	
	waitpid(-1, &status, WNOHANG | WUNTRACED);
}


/*---------------------------------------------------------

	gde_init()

	Initializes Gtk Development Environment. Returns true
	on success, false on error or whenever there is a
	reason to not go into the gtk mainloop.

---------------------------------------------------------*/
static void gde_init(int argc, char *argv[])
{
	int c, opt_index;
	gchar *filename, *cwd;
	gchar last_opened[255];
	
	static struct option long_opts[] = {
		{ "help",		no_argument,	NULL,	'h' },
		{ "version",	no_argument,	NULL,	'v' }
	};

	while (TRUE) {
		c = getopt_long(argc, argv, "hv", long_opts, &opt_index);
		if (c == -1)
			break;

		switch (c) {
		case 'h':
			gde_print_usage();
			break;
		case 'v':
			gde_print_version();
			break;
		default:
			exit(0);
		}
	}

	signal(SIGCHLD, gde_handle_sigchld);

	main_win_create();
	proj_win_create();
	status_win_create();
	
	gde_read_rc();
	
	strcpy(last_opened, gde_rc->last_opened);
	project_new();
	
	if (optind < argc) {
		if (g_path_is_absolute(argv[optind]))
			project_open_file(argv[optind]);
		else {
			cwd = g_get_current_dir();
			filename = g_strconcat(cwd, G_DIR_SEPARATOR_S, argv[optind], NULL);
			project_open_file(filename);
			g_free(filename);
			g_free(cwd);
		}
	}
	else if (gde_rc->open_last_opened)
		project_open_file(last_opened);
}


/*---------------------------------------------------------

	main()

	Here it all starts.	

---------------------------------------------------------*/
int main(int argc, char *argv[])
{
	g_thread_init(NULL);
	gtk_init(&argc, &argv);
	gde_init(argc, argv);

	gdk_threads_enter();
	gtk_main();
	gdk_threads_leave();
	
	return 0;
}
