/*****************************************************************************

	GTK Development Environment
	
	Copyright (c) 2000 End of the World Software

	date:	November 28 2000
	author: Gertjan Zwartjes
	email:	g.zwartjes@student.tue.nl

  **************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "gde.h"
#include "resource/folder.xpm"
#include "resource/ofolder.xpm"
#include "resource/srcfile.xpm"
#include "resource/hdrfile.xpm"
#include "resource/rootfile.xpm"


/*=========================================================

	Variable Declarations

=========================================================*/
static GtkWidget *proj_win;
static GdeWindowInfo *proj_win_info;

static GtkWidget *proj_tree;
static GtkCTreeNode *proj_source_node;
static GtkCTreeNode *proj_header_node;
static GtkCTreeNode *selected_node;

static GdkPixmap *srcfile_pixmap;
static GdkBitmap *srcfile_mask;
static GdkPixmap *hdrfile_pixmap;
static GdkBitmap *hdrfile_mask;
static GdkPixmap *rootfile_pixmap;
static GdkBitmap *rootfile_mask;

enum {
	TARGET_STRING
};

static GtkTargetEntry target_table[] = {
    {"text/uri-list", 0, TARGET_STRING},
    {"STRING", 0, TARGET_STRING},
    {"text/plain", 0, TARGET_STRING}
};
static guint n_targets = sizeof(target_table) / sizeof(target_table[0]);


/*---------------------------------------------------------

	proj_win_drag_data_get()

	Gets the file we are dropping.

---------------------------------------------------------*/
void proj_win_drag_data_get(GtkWidget *widget, GdkDragContext *context,
								   GtkSelectionData *selection_data, guint info,
								   guint time, gpointer data)
{
	GtkCTreeNode *node;
	GdeFileList *file_list;
	gchar file[512];
	
	if(info == TARGET_STRING){
		node = proj_win_selected_node();
		if(node) {
			file_list = project_find_node(node);
			if(file_list) {
				sprintf(file, "file:%s", file_list->filename);
				gtk_selection_data_set(selection_data, selection_data->target, 
					8, file, strlen(file));
			}
		}
	}
}


/*---------------------------------------------------------

	proj_tree_select_row()

	This function is called whenever the user clicks
	on a row of our project tree. If it's a double click,
	we have to load the file into the text editor.

---------------------------------------------------------*/
static void proj_tree_select_row(GtkWidget *widget, gint row, gint column, 
	  					         GdkEventButton *bevent, gpointer data)
{
	if (bevent) {
		selected_node = gtk_ctree_node_nth(GTK_CTREE(widget), row);
		if (bevent->type == GDK_2BUTTON_PRESS)
			project_edit();
	}
}


/*---------------------------------------------------------

	proj_tree_unselect_row()

	This function gets called whenever a row gets
	unselected.

---------------------------------------------------------*/
static void proj_tree_unselect_row(GtkWidget *widget, gint row, gint column)
{
	selected_node = NULL;
}


/*---------------------------------------------------------

	proj_tree_create()

	Creates the project tree in the project window.

---------------------------------------------------------*/
static void proj_tree_create(void)
{
	GdkPixmap *folder_pixmap, *ofolder_pixmap;
	GdkBitmap *folder_mask, *ofolder_mask;

	GtkWidget *proj_scrolled_win;
	gchar* source_files_text = "Source Files";
	gchar* header_files_text = "Header Files";

	proj_scrolled_win = gtk_scrolled_window_new(NULL, NULL);
	
	gtk_container_set_border_width(GTK_CONTAINER (proj_scrolled_win), 0);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (proj_scrolled_win), 
								   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_container_add(GTK_CONTAINER (proj_win), proj_scrolled_win);
	gtk_widget_show(proj_scrolled_win);

	gtk_widget_realize(proj_win);
	folder_pixmap = gdk_pixmap_create_from_xpm_d(proj_win->window, &folder_mask, NULL, folder_xpm);
	ofolder_pixmap = gdk_pixmap_create_from_xpm_d(proj_win->window, &ofolder_mask, NULL, ofolder_xpm);
	srcfile_pixmap = gdk_pixmap_create_from_xpm_d(proj_win->window, &srcfile_mask, NULL, srcfile_xpm);
	hdrfile_pixmap = gdk_pixmap_create_from_xpm_d(proj_win->window, &hdrfile_mask, NULL, hdrfile_xpm);
	rootfile_pixmap = gdk_pixmap_create_from_xpm_d(proj_win->window, &rootfile_mask, NULL, rootfile_xpm);

	proj_tree = gtk_ctree_new(1, 0);
	gtk_clist_set_column_auto_resize(GTK_CLIST(proj_tree), 0, TRUE);
	gtk_clist_set_selection_mode(GTK_CLIST(proj_tree), GTK_SELECTION_SINGLE);
	gtk_ctree_set_line_style(GTK_CTREE(proj_tree), GTK_CTREE_LINES_DOTTED);
	gtk_signal_connect(GTK_OBJECT(proj_tree), "select_row", GTK_SIGNAL_FUNC(proj_tree_select_row), NULL);
	gtk_signal_connect(GTK_OBJECT(proj_tree), "unselect_row", GTK_SIGNAL_FUNC(proj_tree_unselect_row), NULL);
	gtk_container_add(GTK_CONTAINER(proj_scrolled_win), proj_tree);

	gtk_drag_source_set(proj_tree, GDK_BUTTON1_MASK, target_table, n_targets, GDK_ACTION_COPY);
	gtk_signal_connect(GTK_OBJECT (proj_tree), "drag_data_get", GTK_SIGNAL_FUNC(proj_win_drag_data_get), NULL);
	
	proj_source_node = gtk_ctree_insert_node(GTK_CTREE(proj_tree), NULL, NULL, &source_files_text, 4,
											  folder_pixmap, folder_mask, ofolder_pixmap, ofolder_mask,
											  FALSE, FALSE);
	proj_header_node = gtk_ctree_insert_node(GTK_CTREE(proj_tree), NULL, NULL, &header_files_text, 4,
											  folder_pixmap, folder_mask, ofolder_pixmap, ofolder_mask,
											  FALSE, FALSE);
	gtk_widget_show(proj_tree);
}


/*---------------------------------------------------------

	proj_win_create()

	Creates the project window, with GTK+.

---------------------------------------------------------*/
void proj_win_create(void)
{
	proj_win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	
	gtk_signal_connect(GTK_OBJECT (proj_win), "delete_event",
					   GTK_SIGNAL_FUNC (proj_win_hide),
					   NULL);
	gtk_window_set_title(GTK_WINDOW (proj_win), "Project Explorer");
	gtk_widget_set_uposition(proj_win, PROJWIN_LEFT, PROJWIN_TOP);
	gtk_widget_set_usize(proj_win, PROJWIN_WIDTH, PROJWIN_HEIGHT);
	gtk_window_set_policy(GTK_WINDOW (proj_win), FALSE, TRUE, FALSE);
	
	gtk_container_set_border_width(GTK_CONTAINER (proj_win), 2);
	gtk_window_add_accel_group(GTK_WINDOW (proj_win), main_accel_group);

	proj_tree_create();
	
	proj_win_info = g_malloc0(sizeof(GdeWindowInfo));
	gtk_widget_show(proj_win);
}


/*----------------------------------------------------------

	proj_win_destroy()

	Destroy the window and free all memory.

----------------------------------------------------------*/
void proj_win_destroy(void)
{
	g_free(proj_win_info);
	gtk_widget_destroy(proj_win);
}




/*---------------------------------------------------------

	proj_win_show()

	Pop the window on the screen.

---------------------------------------------------------*/
void proj_win_show(void)
{
	if (!GTK_WIDGET_VISIBLE(proj_win)) {
		proj_win_set_window_info(proj_win_info);
	}
}


/*---------------------------------------------------------

	proj_win_hide()

	Remove the window from the screen.

---------------------------------------------------------*/
gint proj_win_hide(void)
{
	if (GTK_WIDGET_VISIBLE(proj_win)) {
		proj_win_get_window_info(proj_win_info);
		gtk_widget_hide(proj_win);
	}
	return TRUE;
}


/*---------------------------------------------------------

	proj_win_add_source_file()

	Add a source files to the project explorer.

---------------------------------------------------------*/
GtkCTreeNode *proj_win_add_source_file(gchar *filename)
{
	GtkCTreeNode *source_file_node;

	source_file_node = gtk_ctree_insert_node(GTK_CTREE(proj_tree), proj_source_node, NULL, &filename, 4,
											 srcfile_pixmap, srcfile_mask, NULL, NULL, FALSE, FALSE);
	return source_file_node;
}


/*---------------------------------------------------------

	proj_win_add_header_file()

	Add a header file to the project tree.

---------------------------------------------------------*/
GtkCTreeNode *proj_win_add_header_file(gchar *filename)
{
	GtkCTreeNode *header_file_node;

	header_file_node = gtk_ctree_insert_node(GTK_CTREE(proj_tree), proj_header_node, NULL, &filename, 4,
											 hdrfile_pixmap, hdrfile_mask, NULL, NULL, FALSE, FALSE);
	return header_file_node;
}


/*---------------------------------------------------------

	proj_win_add_root_file()

	Add a `root' file to the project tree.

---------------------------------------------------------*/
GtkCTreeNode *proj_win_add_root_file(gchar *filename)
{
	GtkCTreeNode *root_file_node;

	root_file_node = gtk_ctree_insert_node(GTK_CTREE(proj_tree), NULL, NULL, &filename, 4,
		rootfile_pixmap, rootfile_mask, NULL, NULL, FALSE, FALSE);
	return root_file_node;
}


/*---------------------------------------------------------

	proj_win_remove_file()

	Remove a file from the project explorer.

---------------------------------------------------------*/
void proj_win_remove_file(GtkCTreeNode *file_node)
{
	gtk_ctree_remove_node(GTK_CTREE(proj_tree), file_node);
	selected_node = NULL;
}


/*---------------------------------------------------------

	proj_win_selected_node()

	Returns the currently selected node.

---------------------------------------------------------*/
GtkCTreeNode *proj_win_selected_node(void)
{
	return selected_node;
}


/*----------------------------------------------------------

	proj_win_get_window_info()

	Return the position, size and visibility.

----------------------------------------------------------*/
void proj_win_get_window_info(GdeWindowInfo* win_info)
{
	win_info->visible = GTK_WIDGET_VISIBLE(proj_win);
	
	if (!win_info->visible) {
		win_info->x = proj_win_info->x;	
		win_info->y = proj_win_info->y;	
		win_info->width = proj_win_info->width;	
		win_info->height = proj_win_info->height;
	}
	else { 
		gdk_window_get_origin(GTK_WIDGET (proj_win)->window, &win_info->x, &win_info->y);
		gdk_window_get_size(GTK_WIDGET (proj_win)->window, &win_info->width, &win_info->height);
	}
}


/*----------------------------------------------------------

	proj_win_set_window_info()

	Set the info of the window.

----------------------------------------------------------*/
void proj_win_set_window_info(GdeWindowInfo *win_info)
{
	if (win_info->visible) {
		if (!GTK_WIDGET_VISIBLE(proj_win))
			gtk_widget_show(proj_win);

		gtk_widget_set_uposition(proj_win, win_info->x, win_info->y);
		gdk_window_resize(proj_win->window, win_info->width, win_info->height);
	}
	else {
		if (GTK_WIDGET_VISIBLE(proj_win))
			proj_win_hide();
		
		proj_win_info->x = win_info->x;		/* Why this? I dunno,  seems like GTK has */
		proj_win_info->y = win_info->y;		/* some bug in displaying  and hiding win */
		proj_win_info->width = win_info->width;		/* Want to know why? Just comment */
		proj_win_info->height = win_info->height;	/* out these four  lines  of code */
		proj_win_info->visible = TRUE;
	}
}
